@@ -1,3 +1,25 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
+   Copyright (C) 2000 CodeFactory AB
+   Copyright (C) 2000 Jonas Borgstr\366m <jonas@codefactory.se>
+   Copyright (C) 2000 Anders Carlsson <andersca@codefactory.se>
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+*/
+
 #include <glib.h>
 #include <string.h>
 
@@ -9,36 +31,44 @@
 
 static HtmlStyle *default_style = NULL;
 
-static gint
-html_style_get_base_size ()
+static GtkStyle *
+html_style_get_gtk_style (void)
 {
-	gint base_size = 14;
-	PangoFontDescription *font_desc;
-	GtkSettings *settings;
-	gchar *font_name;
-	settings = gtk_settings_get_default ();
-	g_object_get (G_OBJECT (settings), "gtk-font-name", &font_name, NULL);
-	font_desc = pango_font_description_from_string (font_name);
-	if (font_desc)
-	  { 
-	    base_size = pango_font_description_get_size (
-		    font_desc)/PANGO_SCALE;
-	  }
-	return base_size;
+  	GtkStyle *style;
+	style = gtk_rc_get_style_by_paths (gtk_settings_get_default(),
+                                           "GtkTextView",
+                                           "TextView",
+                                           HTML_TYPE_VIEW);
+	if (!style) {
+		style = gtk_rc_get_style_by_paths (gtk_settings_get_default(),
+                                                   "GtkHtml",
+                                                   "HtmlView",
+                                                   GTK_TYPE_TEXT_VIEW);
+		if (!style) {
+			style = gtk_style_new ();
+                }
+        }
+
+        return style;
+}
+
+static PangoFontDescription*
+html_style_get_default_font_desc ()
+{
+	GtkStyle *style;
+	style = html_style_get_gtk_style ();
+
+	return style->font_desc;
 }
 
 static HtmlColor*
 html_style_get_default_text_color ()
 {
-	GtkStyle *style = gtk_rc_get_style_by_paths (gtk_settings_get_default(),
-						     "GtkHtml",
-						     "HtmlView",
-						     HTML_TYPE_VIEW);
-	GdkColor text_color;
-	if (!style)
-	  {
-	    style = gtk_style_new ();
-	  }
+	GdkColor  text_color;
+	GtkStyle *style;
+
+        style = html_style_get_gtk_style ();
+
 	text_color = style->text[GTK_STATE_NORMAL];
 
 	return html_color_new_from_rgb (text_color.red,
@@ -182,6 +212,12 @@ html_default_style_new (void)
 	HtmlStyleBackground *background = html_style_background_new ();
 	HtmlStyleBorder *border = html_style_border_new ();
 	HtmlStyleOutline *outline = html_style_outline_new ();
+        gchar *font_name;
+        GtkSettings *settings;
+        PangoFontDescription *font_desc;
+        const gchar *family;
+        gfloat size;
+
 	/* Set refcount to 1 so it never will be deallocated */
 	result->refcount = 1;
 	
@@ -199,19 +235,43 @@ html_default_style_new (void)
 	
 	html_style_set_outline_width (result, HTML_BORDER_WIDTH_MEDIUM);
 
-	/* FIXME: This should probably not be here */
-	inherited->font_spec = html_font_specification_new ("sans", HTML_FONT_STYLE_NORMAL, 
-							    HTML_FONT_VARIANT_NORMAL, 
-							    HTML_FONT_WEIGHT_NORMAL, 
-							    HTML_FONT_STRETCH_NORMAL,
-							    HTML_FONT_DECORATION_NONE, html_style_get_base_size ());
-
+	font_desc = html_style_get_default_font_desc ();
+        
+	family = pango_font_description_get_family (font_desc);
+	size = pango_font_description_get_size (font_desc) / (gfloat)PANGO_SCALE;
+        
+	inherited->font_spec = 
+                html_font_specification_new ((gchar *) family, 
+                                             HTML_FONT_STYLE_NORMAL, 
+                                             HTML_FONT_VARIANT_NORMAL, 
+                                             HTML_FONT_WEIGHT_NORMAL, 
+                                             HTML_FONT_STRETCH_NORMAL,
+                                             HTML_FONT_DECORATION_NONE, 
+                                             size);
+	
 	/* Set the default color to match the GTK+ theme */
 	html_style_set_color (result, html_style_get_default_text_color ());
 	
 	return result;
 }
 
+static void
+html_style_notify_settings (GObject *obj, GParamSpec *pspec)
+{
+	if (strcmp (pspec->name, "gtk-theme-name") == 0) {
+		HtmlColor *color;
+
+		/*
+		 * Update the default color
+		 */
+		color =  html_style_get_default_text_color ();
+		default_style->inherited->color->red =  color->red;
+		default_style->inherited->color->green =  color->green;
+		default_style->inherited->color->blue =  color->blue;
+		g_free (color);
+	}
+}
+                            
 HtmlStyle *
 html_style_new (HtmlStyle *parent)
 {
@@ -219,6 +279,10 @@ html_style_new (HtmlStyle *parent)
 
 	if (!default_style) {
 		default_style = html_default_style_new ();
+		g_signal_connect (gtk_settings_get_default (),
+				  "notify",
+				  G_CALLBACK (html_style_notify_settings),
+				  NULL);
 	}
 
 	/* FIXME: Eeeek, very ugly */
